package morpion.modele.plateau;

import java.util.ArrayList;
import java.util.List;

/**
 * Un plateau de jeu de morpion. Cet objet représente simplement un plateau de
 * jeu, sur lequel des pions sont disposés. Il ne connaît pas les règles du jeu,
 * si bien qu'on peut l'initialiser à sa guise.
 *
 * @author rosmord
 */
public class PlateauMorpionModele {

    private ContenuCase[][] cases;

    private List<PlateauMorpionListener> plateauMorpionListeners = new ArrayList<>();

    public PlateauMorpionModele() {
        cases = new ContenuCase[3][3];
        for (int i = 0; i < cases.length; i++) {
            for (int j = 0; j < cases[i].length; j++) {
                cases[i][j] = ContenuCase.VIDE;
            }
        }
    }

    /**
     * Fixe le contenu d'une case.
     *
     * @param i compris entre 0 et 2
     * @param j compris entre 0 et 2
     * @param contenuCase
     * @throws ArrayIndexOutOfBoundsException si i ou j sont hors limites.
     */
    public void deposer(int i, int j, ContenuCase contenuCase) {
        cases[i][j] = contenuCase;
        // PEUT ÊTRE DU CODE ICI ???
    }

    /**
     * Récupère le contenu d'une case.
     * <p>
     * Pour simplifier les calculs lors de la détection d'alignement, toutes les
     * cases "hors du plateau", c'est à dire dont les coordonnées ne sont pas
     * comprises entre 0 et 2 (inclus) sont considérées comme vide, et ne
     * déclenchent pas d'exception.
     *
     * @param i
     * @param j
     * @return
     */
    public ContenuCase get(int i, int j) {
        if (i < 0 || i >= 3 || j < 0 || j >= 3) {
            return ContenuCase.VIDE;
        } else {
            return cases[i][j];
        }
    }

    public boolean estVide(int i, int j) {
        return cases[i][j] == ContenuCase.VIDE;
    }

    /**
     * Renvoie la liste du contenu des différentes lignes.
     *
     * @return
     */
    public List<List<ContenuCase>> getLignes() {
        ArrayList<List<ContenuCase>> result = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            ArrayList<ContenuCase> l = new ArrayList<>();
            for (int j = 0; j < 3; j++) {
                l.add(get(i, j));
            }
            result.add(l);
        }
        return result;
    }

    /**
     * Renvoie la liste du contenu des différentes colonnes.
     *
     * @return
     */
    public List<List<ContenuCase>> getColonnes() {
        ArrayList<List<ContenuCase>> result = new ArrayList<>();
        for (int j = 0; j < 3; j++) {
            ArrayList<ContenuCase> l = new ArrayList<>();
            for (int i = 0; i < 3; i++) {
                l.add(get(i, j));
            }
            result.add(l);
        }
        return result;
    }

    /**
     * Renvoie la liste du contenu des différentes diagonales.
     *
     * @return
     */
    public List<List<ContenuCase>> getDiagonales() {
        ArrayList<List<ContenuCase>> result = new ArrayList<>();
        ArrayList<ContenuCase> d1 = new ArrayList<>();
        ArrayList<ContenuCase> d2 = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            d1.add(get(i, i));
            d2.add(get(i, 2 - i));
        }
        result.add(d1);
        result.add(d2);
        return result;
    }

    public void addPlateauMorpionListener(PlateauMorpionListener l) {
        plateauMorpionListeners.add(l);
    }

    public void removePlateauMorpionListener(PlateauMorpionListener l) {
        plateauMorpionListeners.remove(l);
    }

    public void notifyPlateauMorpionListeners() {
        for (PlateauMorpionListener l : plateauMorpionListeners) {
            l.plateauModifie();
        }
    }

    public Case getCase(int i, int j) {
        return new Case(i, j, get(i, j));
    }

    /**
     * Retourne vrai s'il n'y a plus de case libre sur le plateau.
     * @return 
     */
    public boolean estPlein() {
        // On vérifie si le tableau est plein:
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (estVide(i, j)) {
                    // On a trouvé une case libre,
                    // le plateau n'est pas plein !
                    return false;
                }
            }
        }
        return true;
    }

}
